#!BPY
""" Registration info for Blender menus
Name: 'ph_BandGlue'
Blender: 231
Group: 'Misc'
Tooltip: 'Colle une bande de faces.'
"""

#### !<>

########################################################
#													   #
#				 ph_BandGlue.py						   #
#													   #
#		  Cree par coolphil@worldonline.fr			   #
#				  le 14/05/2004						   #
#													   #
#		  	Outil d'optimisation de mesh.			   #
#													   #
# http://hp.home.chez.tiscali.fr/re_illusion/index.htm #
#													   #
########################################################


import Blender, time
from Blender import Scene, Object, NMesh, Window
from Blender.Draw import *
from Blender.BGL import *
from math import *


#### Global Variables #############

###### Quelques couleurs #######
white = (1.0,1.0,1.0)
black = (0.0,0.0,0.0)
blue = (.71,.71,.76)
dgray = (.4,.4,.4)
red = (.7,.0,.3)
green = (.70,.75,.70)
back = (.7,.7,.7)
################################



H_Col = blue
Border = (H_Col[0]-.2,H_Col[1]-.2,H_Col[2]-.2)
Glanz = (H_Col[0]+.2,H_Col[1]+.2,H_Col[2]+.2)

BWidth = 230
BHeight = 130

W_up = Create(0)
W_down = Create(1)
W_left = Create(1)
W_right = Create(0)

lastUndo = []
warn = False
start = 0.0
end = 0.0
DelayWanted = 0.08



EVENT_EXIT = 1
EVENT_WPOS_U = 2
EVENT_WPOS_D = 3
EVENT_WPOS_L = 4
EVENT_WPOS_R = 5
EVENT_ACTION = 6
EVENT_UNDO = 7

###################################



BL_VERSION = Blender.Get('version')
if (BL_VERSION<=223):
	import Blender210


################
def GL_Bx(vl=0):
################
	return ((int(WindowSize[2] - BWidth) * W_right.val) + vl)

################
def GL_By(vl=0):
################
	return ((int(WindowSize[3] - BHeight) * W_up.val) + vl)

###################
def GL_B_4dir(x,y):
###################
	global W_up, W_down, W_left, W_right

	W_up = Toggle("", EVENT_WPOS_U, GL_Bx(x+9), GL_By(y+15), 7, 7, W_up.val, "Move to the up of the window")
	W_down = Toggle("", EVENT_WPOS_D, GL_Bx(x+9), GL_By(y-1), 7, 7, W_down.val, "Move to the down of the window")
	W_left = Toggle("", EVENT_WPOS_L, GL_Bx(x), GL_By(y+7), 7, 7, W_left.val, "Move to the left of the window")
	W_right = Toggle("", EVENT_WPOS_R, GL_Bx(x+18), GL_By(y+7), 7, 7, W_right.val, "Move to the right of the window")

	return 0

#############################
def Ligne(y,xs=20,xe=BWidth):
#############################
	glRecti(xs,y,xe,y+1)

######################################
def Ligne3D(y,long,col,flat=20,xs=20):
######################################
	t= flat/100.0
	glColor3d(col[0]+t,col[1]+t,col[2]+t)
	Ligne(y,xs,xs+long)
	glColor3d(col[0]-t,col[1]-t,col[2]-t)
	Ligne(y+1,xs,xs+long)
	
#############################################################################
def Degrade(y,height,col,flat=10,xs=20,xe=BWidth,half=0,bizS=0,bizE=0):
#############################################################################
	if not((bizS == 0) or (bizS == -1) or (bizS == 1)):
		bizS = 0
 	if not((bizE == 0) or (bizE == -1) or (bizE == 1)):
		bizE = 0


	cC = 0
	cT = 1+half
	for cl in range(0, height, (1 - 2 * (height < 0))):
		incS = xs + ( height * cl/height * bizS)
		incE = xe + ( height * cl/height * bizE)

		cC = -cos(cl/float(height)*pi/(cT+(cT==0)))*flat/100
		glColor3d(col[0]+cC,col[1]+cC,col[2]+cC)
		Ligne(y+cl,incS,incE)


##################################################
def GL_Dot(x=10, y=10, col=(.71,.71,.76), flat=2):
##################################################
	DotList = ((-.05,-.1,-.1),(.05,.0,-.1),(.1,.05,-.05))
	glBegin(GL_POINTS)
	for ze in range(3):
		for sp in range(3):
			cC = DotList[ze][sp]*flat
			glColor3d(col[0]+cC,col[1]+cC,col[2]+cC)
			glVertex2i(x+sp,y+ze)
	glEnd()

##################################################################################
def GL_DotLine(x=10, y=10, long = 10, col=(.71,.71,.76), direction = "y", flat=5):
##################################################################################

	if direction == "x":
		for Dot_X in range(x, x+long, 10):
			GL_Dot(Dot_X, y, col, flat)

	else:
		for Dot_Y in range(y, y+long, 10):
			GL_Dot(x, Dot_Y, col, flat)

	return 0

########################################################################
def Relief_Text(x,y,t, colfrt=(1.0, 1.0, 1.0), colback=(0.0, 0.0, 0.0)):
########################################################################

	apply(glColor3f, colback)
	glRasterPos2f(x+1, y+3)
	Text(t)
	glRasterPos2f(x+1, y+4)
	Text(t)
	apply(glColor3f, colfrt)
	glRasterPos2f(x, y+5)
	Text(t)

	return 0


###############
def info_Obj(Ob=[]):
###############

	Obtype = 0
	mename = []
	me = []
	calque = 1

	if not Ob:
		if not Object.GetSelected():
			return Ob,Obtype,mename,me
		else:
			Ob = Object.GetSelected()[0]
			if (Ob.getType() == 'Mesh'):

				Obtype = 1
				calque = Ob.Layer                    # Le "layer" de l'object selectionne
				mename = Ob.getData().name
				me = Ob.getData()
	else:
		if (Ob.getType() == 'Mesh'):

			Obtype = 1
			calque = Ob.Layer                    # Le "layer" de l'object selectionne
			mename = Ob.getData().name
			me = Ob.getData()

	return Ob,Obtype,mename,me

#########################################
def D_Faces_f_VertAndD_Edges_f_Face(me):#
#########################################
# Dicos sous forme de NMesh.verts #
###################################

	D_ffv = {} ### DicoFacesfromVert={[NMVert]: [[NMFace],[NMFace],[NMFace]],  [NMVert]: [[NMFace],[NMFace]],  [NMVert]:  ...}
	D_eff = {} ### DicoEdgesfromFace={[NMFace]: [[NMVert,NMVert], [NMVert,NMVert],[NMVert,NMVert],[NMVert,NMVert]],  [NMFace]: [[NMVert,NMVert], [NMVert, ...}


	for x in me.verts:
		D_ffv[x] = []

	MeshFaces = me.faces

	for f in MeshFaces:
		lenF = len(f)

		D_eff[f] = []

		if lenF == 2:
			D_eff[f] = [f.v]

		else:

			for vidx in range(lenF):

				###### Section L_Ffv ##########
				if f not in D_ffv[f.v[vidx]]:
					D_ffv[f.v[vidx]].append(f)

				###### Section D_eff #########
				if vidx < (lenF-1):
					D_eff[f].append([f.v[vidx], f.v[vidx+1]])
				else:
					D_eff[f].append([f.v[vidx], f.v[0]])


	return D_ffv, D_eff

############################################################################
def ReturnFacesfromEdge(edge=[None,None], d_ffv={}, d_eff={}, selfFace=[]):#
############################################################################
# Retourne la liste des faces qui appartiennent a un edge donne #
#################################################################

#	print "1er vertice de l'edge  = ", edge[0]
#	print "2eme vertice de l'edge  = ", edge[1]

	if len(edge) > 2:
		error = "Error, an edge can be made by two and only two points !"
		raise error

	else:
#		print "d_ffv[edge[0]] = ", d_ffv[edge[0]]

		l_fcs = []
		[l_fcs.append(x) for x in d_ffv[edge[0]]]
		[l_fcs.append(x) for x in d_ffv[edge[1]]]
		reduce(l_fcs)

#		print "==== l_fcs =", l_fcs

		if selfFace:
			tp1 = []
			[tp1.append(x) for x in l_fcs]
			tp1.remove(selfFace)
			l_fcs = tp1

		l_ffe = []
		[l_ffe.append(f) for f in l_fcs if edge in d_eff[f] or (Swap2d(edge) in d_eff[f])]

		if l_ffe:
			if len(l_ffe) == 1:
				l_ffe = l_ffe[0]


	return l_ffe

#############################
def Swap2d(list=[None,None]):
#############################

	return [list[1], list[0]]


#######################
#### Fonction reduce ##
def reduce(list):     #
#######################

	list.sort()
	list.reverse()

	lenlist = len(list) - 1
	if lenlist > 0:
		compt = 0

		while compt < lenlist:

			if list[compt] == list[compt + 1]:
				list.remove(list[compt])
				lenlist -= 1

			else:
				compt += 1

	list.reverse()

	return list

######################
def Copy(ToCopy = []):
######################

	A = []
	[A.append(x) for x in ToCopy]

	return A

##########################
def GetOpenGLFaceImage(f):
##########################

	img = []

	if f.image:

			img = f.image


	return img


#####################################
def AttributFace(fc, me, pos1, pos2):
#####################################

	attr_fc = [[], [], [], [], [], [], []]

	attr_fc[0] = [fc.smooth]

	if fc.col:
		attr_fc[1] = [fc.col[pos1], fc.col[pos2]] ### Vertex color de la face

		attr_fc[2] = [fc.uv[pos1], fc.uv[pos2]] ### UVs de la face

		tpimg = GetOpenGLFaceImage(fc)
		attr_fc[3] = tpimg ### Image_UV de la face

		attr_fc[4] = fc.mode ### mode de la face

		attr_fc[5] = fc.transp ### transparence_mode de la face

	if me.materials:
		attr_fc[6] = fc.materialIndex ### copy du materialIndex de la face

	return attr_fc

#############################
def Ret3_OtherEdges(edg, fc):
#############################


				#### On verifier l'orientation de sa normale pour trouver le prochain edge ####
				p1 = fc.v.index(edg[0])
				p2 = fc.v.index(edg[1])

				if (p1==1 and p2==0) or (p1==2 and p2==1) or (p1==3 and p2==2) or (p1==0 and p2==3):

					#### Orientation OK ####

					if (p1==1 and p2==0):
						next1 = [fc.v[1], fc.v[2]]
						next2 = [fc.v[2], fc.v[3]]
						next3 = [fc.v[3], fc.v[0]]

					elif (p1==2 and p2==1):
						next1 = [fc.v[2], fc.v[3]]
						next2 = [fc.v[3], fc.v[0]]
						next3 = [fc.v[0], fc.v[1]]

					elif (p1==3 and p2==2):
						next1 = [fc.v[3], fc.v[0]]
						next2 = [fc.v[0], fc.v[1]]
						next3 = [fc.v[1], fc.v[2]]

					elif (p1==0 and p2==3):
						next1 = [fc.v[0], fc.v[1]]
						next2 = [fc.v[1], fc.v[2]]
						next3 = [fc.v[2], fc.v[3]]


				elif (p1==0 and p2==1) or (p1==1 and p2==2) or (p1==2 and p2==3) or (p1==3 and p2==0):

					#### Orientation inverse ####

					if (p1==0 and p2==1):
						next1 = [fc.v[0], fc.v[3]]
						next2 = [fc.v[3], fc.v[2]]
						next3 = [fc.v[2], fc.v[1]]

					elif (p1==1 and p2==2):
						next1 = [fc.v[1], fc.v[0]]
						next2 = [fc.v[0], fc.v[3]]
						next3 = [fc.v[3], fc.v[2]]

					elif (p1==2 and p2==3):
						next1 = [fc.v[2], fc.v[1]]
						next2 = [fc.v[1], fc.v[0]]
						next3 = [fc.v[0], fc.v[3]]

					elif (p1==3 and p2==0):
						next1 = [fc.v[3], fc.v[2]]
						next2 = [fc.v[2], fc.v[1]]
						next3 = [fc.v[1], fc.v[0]]


				return next1, next2, next3

###########
def Undo():
###########
	global Obj, mesh, meshname, lastUndo

	NMesh.PutRaw(lastUndo, meshname)
	Obj.makeDisplayList()

	lastUndo = []

	return 0


###########
def main():
###########
	global Obj, mesh, meshname, lastUndo, warn, start

	Obj,Objtype,meshname,mesh = info_Obj()

	if Objtype:
		if len(mesh.getSelectedFaces()) > 1  and not mesh.getVertGroupNames():


			print
			print "Working..."

			lastUndo = Obj.getData()

			SelFcs = mesh.getSelectedFaces()
			AttrFcs = {}
			NextFc2Check = []
			BandFcs = {}
			ComptBand = 0
			EvenStart = {}
			EvenEnd = {}
			CurEdge = []
			EdgesNwFcs = {}
			listVtx = []
			Vtx2Keep = []
			l_limite1 = []
			l_limite2 = []
			H_limite1 = False
			H_limite2 = False

			#####################################
			#####################################
			print "Building ... DicoFacesfromVert"
			D_FacesfromVert, D_EdgesfromFace = D_Faces_f_VertAndD_Edges_f_Face(mesh)
			print "DicoFacesfromVert...OK"
			#####################################


			#############################################################################################
			######## Recherche de la premiere face qui se trouve a une extremite de la selection ########


			SelectFaces = mesh.getSelectedFaces()
			BandFcs[ComptBand] = []

			for f in SelectFaces:

				if not len(f.v) == 4:
					warn = True
					start = time.clock()
					print
					print "================================"
					print "Selected faces have to be quad !"
					print "================================"
					return 0

				edgs = D_EdgesfromFace[f]
				OddFfE1 = ReturnFacesfromEdge(edgs[0], D_FacesfromVert, D_EdgesfromFace, f)
				OddFfE2 = ReturnFacesfromEdge(edgs[2], D_FacesfromVert, D_EdgesfromFace, f)
				EvenFfE1 = ReturnFacesfromEdge(edgs[1], D_FacesfromVert, D_EdgesfromFace, f)
				EvenFfE2 = ReturnFacesfromEdge(edgs[3], D_FacesfromVert, D_EdgesfromFace, f)


				if not ((OddFfE1 in SelFcs) and (OddFfE2 in SelFcs)) or((EvenFfE1 in SelFcs) and (EvenFfE2 in SelFcs)):

					#### La face est sur l'extremite de la selection ####

					if (OddFfE1 in SelFcs) and (OddFfE2 not in SelFcs):
						#### Sens de depart trouve ####

						AttrFcs[ComptBand] = AttributFace(f, mesh, 2, 3)

						EvenStart[f] = edgs[3]
						EvenEnd[f] = edgs[1]
						CurEdge = Copy(edgs[0])
						listVtx.extend(edgs[0])
						l_limite1 = Copy(edgs[2])
						Vtx2Keep.extend(edgs[2])
						EdgesNwFcs[ComptBand] = []
						EdgesNwFcs[ComptBand].extend(edgs[2])
						BandFcs[ComptBand].append(f)
						NextFc2Check = OddFfE1

						break #### On peut arreter la boucle car on a trouve notre face de depart ####


					elif (OddFfE2 in SelFcs) and (OddFfE1 not in SelFcs):
						#### Sens de depart trouve ####

						AttrFcs[ComptBand] = AttributFace(f, mesh, 0, 1)

						EvenStart[f] = edgs[1]
						EvenEnd[f] = edgs[3]
						CurEdge = Copy(edgs[2])
						listVtx.extend(edgs[2])
						l_limite1 = Copy(edgs[0])
						Vtx2Keep.extend(edgs[0])
						EdgesNwFcs[ComptBand] = []
						EdgesNwFcs[ComptBand].extend(edgs[0])
						BandFcs[ComptBand].append(f)
						NextFc2Check = OddFfE2

						break #### On peut arreter la boucle car on a trouve notre face de depart ####


					elif (EvenFfE1 in SelFcs) and (EvenFfE2 not in SelFcs):
						#### Sens de depart trouve ####

						AttrFcs[ComptBand] = AttributFace(f, mesh, 3, 0)

						EvenStart[f] = edgs[0]
						EvenEnd[f] = edgs[2]
						CurEdge = Copy(edgs[1])
						listVtx.extend(edgs[1])
						l_limite1 = Copy(edgs[3])
						Vtx2Keep.extend(edgs[3])
						EdgesNwFcs[ComptBand] = []
						EdgesNwFcs[ComptBand].extend(edgs[3])
						BandFcs[ComptBand].append(f)
						NextFc2Check = EvenFfE1

						break #### On peut arreter la boucle car on a trouve notre face de depart ####


					elif (EvenFfE2 in SelFcs) and (EvenFfE1 not in SelFcs):
						#### Sens de depart trouve ####

						AttrFcs[ComptBand] = AttributFace(f, mesh, 1, 2)

						EvenStart[f] = edgs[2]
						EvenEnd[f] = edgs[0]
						CurEdge = Copy(edgs[3])
						listVtx.extend(edgs[3])
						l_limite1 = Copy(edgs[1])
						Vtx2Keep.extend(edgs[1])
						EdgesNwFcs[ComptBand] = []
						EdgesNwFcs[ComptBand].extend(edgs[1])
						BandFcs[ComptBand].append(f)
						NextFc2Check = EvenFfE2

						break #### On peut arreter la boucle car on a trouve notre face de depart ####


			###### Recherche OK #######
			###### suite #####

			if not BandFcs[ComptBand]:
				warn = True
				start = time.clock()
				print "============================="
				print "La selection n'est pas aligne"
				print "============================="
				return 0


			if not NextFc2Check in SelFcs:
				warn = True
				start = time.clock()
				print "============================="
				print "La selection n'est pas aligne"
				print "============================="
				return 0 #### Selection non alignee ####

			RestOfSelectFcs = Copy(SelFcs)
			RestOfSelectFcs.remove(BandFcs[ComptBand][0])



			##### On va ordonner le reste des selected Faces #####

			lenSelF = len(SelFcs) #### nombre de selectedFaces ####
			UVend = []

			Breakloop = 0
			comptFcs = 0


			####################
			while not Breakloop:

				if not NextFc2Check in RestOfSelectFcs:
					warn = True
					start = time.clock()
					print "============================="
					print "La selection n'est pas aligne"
					print "============================="
					return 0 #### Selection non alignee ####
				else:
					RestOfSelectFcs.remove(NextFc2Check) #### OK, face accepte ####

				#### On verifie que c'est un quad ####
				if len(NextFc2Check.v) != 4:
					warn = True
					start = time.clock()
					print
					print "================================"
					print "Selected faces have to be quad !"
					print "================================"
					return 0

				CurFace = NextFc2Check
				BandFcs[ComptBand].append(CurFace)
				comptFcs += 1

				###################################################################################
				#### On verifier l'orientation de sa normale pour trouver les 3 edges suivants ####
				tpevs, tpcuredg, tpeve = Ret3_OtherEdges(CurEdge, CurFace)
				EvenStart[CurFace] = tpevs
				CurEdge = tpcuredg
				EvenEnd[CurFace] = tpeve
				###################################################################################


				NextFc2Check = ReturnFacesfromEdge(CurEdge, D_FacesfromVert, D_EdgesfromFace, CurFace)


				if RestOfSelectFcs: #### Tant qu'on est pas sur la derniere face a controler ####
					if (NextFc2Check in RestOfSelectFcs):
						#### La face n'est pas sur l'extremite de la selection mais dans son milieu ####
						listVtx.extend(CurEdge)
						pass #### On poursuit avec la face suivante ####

					else:
						#### La selection n'est pas aligne ####
						warn = True
						start = time.clock()
						print "============================="
						print "La selection n'est pas aligne"
						print "============================="
						return 0


				else: #### On est sur la derniere face a controler ####
					l_limite2 = CurEdge
					EdgesNwFcs[ComptBand].extend(CurEdge)

					colEnd = [CurFace.col[CurFace.v.index(CurEdge[0])], CurFace.col[CurFace.v.index(CurEdge[1])]]
					UVend = [CurFace.uv[CurFace.v.index(CurEdge[0])], CurFace.uv[CurFace.v.index(CurEdge[1])]]
					Vtx2Keep.extend(CurEdge)
					Breakloop = 1


			##### Out of While #####

			if not l_limite2:
				return 0


			NewCol = [AttrFcs[ComptBand][1][0], AttrFcs[ComptBand][1][1], colEnd[0], colEnd[1]]
			NewUV = [AttrFcs[ComptBand][2][0], AttrFcs[ComptBand][2][1], UVend[0], UVend[1]]
			AttrFcs[ComptBand][1] = NewCol
			AttrFcs[ComptBand][2] = NewUV



			##### Recherche de H_limite1 ou tour complet #################################################

			finish = 0
			ComptBand += 1

			############################################################################################################
			while not finish:

				BandFcs[ComptBand] = [] #### On prepare la prochaine band ####

				for fidx in range(len(BandFcs[ComptBand - 1])):

					#### On fait une nouvelle liste de (Sel)faces ####

					f = BandFcs[ComptBand - 1][fidx]
					edg = EvenStart[BandFcs[ComptBand - 1][fidx]]

					retF = ReturnFacesfromEdge(edg, D_FacesfromVert, D_EdgesfromFace, f)
					if retF:
						if len(retF.v) != 4:
							warn = True
							start = time.clock()
							print "==============="
							print "Tris decouvert."
							print "==============="
							return 0 #### Tris decouvert ####

						BandFcs[ComptBand].append(retF)

				#### La nouvelle liste de (Sel)faces est construite ####

			##############
				if len(BandFcs[ComptBand]) != len(BandFcs[0]):
					if BandFcs[ComptBand]: #### La limite n'est pas franche ####
						warn = True
						start = time.clock()
						print "============="
						print "Can't do it !"
						print "============="
						return 0

					else: #### H_limite1 atteinte ####
						H_limite1 = True
						finish = 1
			##############

				elif BandFcs[ComptBand] == BandFcs[0]: #### Un tour complet ####
					#### Finir les derniers details puis ... ####
					finish = 1

				else: #### Quelques details puis On passe a la band suivante jusqu'a H_limite1 ou tour complet ####

					for fidx in range(len(BandFcs[ComptBand])):

			####### OPTION : Selection des faces avant suppression ########
						mesh.faces[mesh.faces.index(BandFcs[ComptBand][fidx])].flag = 1
			###############################################################


						#######################
						tp_next1, tp_next2, tp_next3 = Ret3_OtherEdges(EvenStart[BandFcs[ComptBand - 1][fidx]], BandFcs[ComptBand][fidx])
						#######################

						if not fidx: #### 1ere face de la band ####
							Vtx2Keep.extend(tp_next1)
							listVtx.extend(tp_next3)
							EdgesNwFcs[ComptBand] = []
							EdgesNwFcs[ComptBand].extend(tp_next1)
							AttrFcs[ComptBand] = AttributFace(BandFcs[ComptBand][fidx], mesh, BandFcs[ComptBand][fidx].v.index(tp_next1[0]), BandFcs[ComptBand][fidx].v.index(tp_next1[1]))

							EvenStart[BandFcs[ComptBand][fidx]] = tp_next2

						elif fidx == (len(BandFcs[ComptBand]) - 1): #### Derniere face de la band ####
							if not ((tp_next1 == tp_prec3) or (tp_next1 == Swap2d(tp_prec3)) or (tp_next1 == tp_prec1) or (tp_next1 == Swap2d(tp_prec1)) or (tp_next3 == tp_prec3) or (tp_next3 == Swap2d(tp_prec3)) or (tp_next3 == tp_prec1) or (tp_next3 == Swap2d(tp_prec1))):
								warn = True
								start = time.clock()
								print "============="
								print "Can't do it !"
								print "============="
								return 0
							Vtx2Keep.extend(tp_next3)
							EdgesNwFcs[ComptBand].extend(tp_next3)
							tpattr = AttributFace(BandFcs[ComptBand][fidx], mesh, BandFcs[ComptBand][fidx].v.index(tp_next3[0]), BandFcs[ComptBand][fidx].v.index(tp_next3[1]))

							NewCol = AttrFcs[ComptBand][1]
							NewCol.extend(tpattr[1])
							NewUV = AttrFcs[ComptBand][2]
							NewUV.extend(tpattr[2])

							EvenStart[BandFcs[ComptBand][fidx]] = tp_next2

						else:
							if not ((tp_next1 == tp_prec3) or (tp_next1 == Swap2d(tp_prec3)) or (tp_next1 == tp_prec1) or (tp_next1 == Swap2d(tp_prec1)) or (tp_next3 == tp_prec3) or (tp_next3 == Swap2d(tp_prec3)) or (tp_next3 == tp_prec1) or (tp_next3 == Swap2d(tp_prec1))):
								warn = True
								start = time.clock()
								print "============="
								print "Can't do it !"
								print "============="
								return 0
							listVtx.extend(tp_next3)

							EvenStart[BandFcs[ComptBand][fidx]] = tp_next2

						tp_prec1 = tp_next1
			#			tp_prec2 = tp_next2
						tp_prec3 = tp_next3


				ComptBand += 1




			NotFirstTime = 0
			finish = 0
			if H_limite1:
				ComptBand -= 1
				##### Recherche de H_limite2 #################################################
				####################################################################################################################################################
				while not finish:

					BandFcs[ComptBand] = [] #### On prepare la prochaine band ####

					for fidx in range(len(BandFcs[(ComptBand - 1) * NotFirstTime])):

						#### On fait une nouvelle liste de (Sel)faces ####

						f = BandFcs[(ComptBand - 1) * NotFirstTime][fidx]
						edg = EvenEnd[BandFcs[(ComptBand - 1) * NotFirstTime][fidx]]

						retF = ReturnFacesfromEdge(edg, D_FacesfromVert, D_EdgesfromFace, f)
						if retF:
							if len(retF.v) != 4:
								warn = True
								start = time.clock()
								print "==============="
								print "Tris decouvert."
								print "==============="
								return 0 #### Tris decouvert ####

							BandFcs[ComptBand].append(retF)

					#### La nouvelle liste de (Sel)faces est construite ####


				##############
					if len(BandFcs[ComptBand]) != len(BandFcs[0]):
						if BandFcs[ComptBand]: #### La limite n'est pas franche ####
							warn = True
							start = time.clock()
							print "============="
							print "Can't do it !"
							print "============="
							return 0
						else: #### H_limite2 atteinte ####
							H_limite2 = True
							finish = 1
				##############



					else: #### Quelques details puis On passe a la band suivante jusqu'a H_limite2 ####

						for fidx in range(len(BandFcs[ComptBand])):

				####### OPTION : Selection des faces avant suppression ########
							mesh.faces[mesh.faces.index(BandFcs[ComptBand][fidx])].flag = 1
				###############################################################


							#######################
							tp_next1, tp_next2, tp_next3 = Ret3_OtherEdges(EvenEnd[BandFcs[(ComptBand - 1) * NotFirstTime][fidx]], BandFcs[ComptBand][fidx])
							#######################
							if not fidx: #### 1ere face de la band ####
								Vtx2Keep.extend(tp_next3)
								listVtx.extend(tp_next1)
								EdgesNwFcs[ComptBand] = []
								EdgesNwFcs[ComptBand].extend(tp_next3)
								AttrFcs[ComptBand] = AttributFace(BandFcs[ComptBand][fidx], mesh, BandFcs[ComptBand][fidx].v.index(tp_next3[0]), BandFcs[ComptBand][fidx].v.index(tp_next3[1]))

								EvenEnd[BandFcs[ComptBand][fidx]] = tp_next2

							elif fidx == (len(BandFcs[ComptBand]) - 1): #### Derniere face de la band ####
								if not ((tp_next1 == tp_prec3) or (tp_next1 == Swap2d(tp_prec3)) or (tp_next1 == tp_prec1) or (tp_next1 == Swap2d(tp_prec1)) or (tp_next3 == tp_prec3) or (tp_next3 == Swap2d(tp_prec3)) or (tp_next3 == tp_prec1) or (tp_next3 == Swap2d(tp_prec1))):
									warn = True
									start = time.clock()
									print "============="
									print "Can't do it !"
									print "============="
									return 0
								Vtx2Keep.extend(tp_next1)
								EdgesNwFcs[ComptBand].extend(tp_next1)
								tpattr = AttributFace(BandFcs[ComptBand][fidx], mesh, BandFcs[ComptBand][fidx].v.index(tp_next1[0]), BandFcs[ComptBand][fidx].v.index(tp_next1[1]))

								NewCol = AttrFcs[ComptBand][1]
								NewCol.extend(tpattr[1])
								NewUV = AttrFcs[ComptBand][2]
								NewUV.extend(tpattr[2])

								EvenEnd[BandFcs[ComptBand][fidx]] = tp_next2

							else:
								if not ((tp_next1 == tp_prec3) or (tp_next1 == Swap2d(tp_prec3)) or (tp_next1 == tp_prec1) or (tp_next1 == Swap2d(tp_prec1)) or (tp_next3 == tp_prec3) or (tp_next3 == Swap2d(tp_prec3)) or (tp_next3 == tp_prec1) or (tp_next3 == Swap2d(tp_prec1))):
									warn = True
									start = time.clock()
									print "============="
									print "Can't do it !"
									print "============="
									return 0
								listVtx.extend(tp_next1)

								EvenEnd[BandFcs[ComptBand][fidx]] = tp_next2

							tp_prec1 = tp_next1
				#			tp_prec2 = tp_next2
							tp_prec3 = tp_next3


					if not NotFirstTime:
						NotFirstTime = 1

					ComptBand += 1
				############################################################################################################################################################
			ComptBand -= 1

			################# Suppression des faces et vertex #########################

			listVtx.sort()
			reduce(listVtx)

			for idx in range(ComptBand):
				for f in BandFcs[idx]:
					del(mesh.faces[mesh.faces.index(f)])

			for v in listVtx:
				del(mesh.verts[mesh.verts.index(v)])


			################# Creation des nouvelles faces #######################

			NewFcsIdx = []
			for idx in range(ComptBand):
				NewFcsIdx.append(len(mesh.faces))

				NewFace = NMesh.Face()

				NewFace.v.append(EdgesNwFcs[idx][0])
				NewFace.v.append(EdgesNwFcs[idx][1])
				NewFace.v.append(EdgesNwFcs[idx][2])
				NewFace.v.append(EdgesNwFcs[idx][3])


				mesh.faces.append(NewFace)


			mesh.update()

			for idx in range(ComptBand):

				mesh.faces[NewFcsIdx[idx]].smooth = AttrFcs[idx][0][0]
				mesh.faces[NewFcsIdx[idx]].col = AttrFcs[idx][1]
				mesh.faces[NewFcsIdx[idx]].uv = AttrFcs[idx][2]

				if AttrFcs[idx][3]:
					mesh.faces[NewFcsIdx[idx]].image = AttrFcs[idx][3]

				mesh.faces[NewFcsIdx[idx]].mode = AttrFcs[idx][4]
				mesh.faces[NewFcsIdx[idx]].transp = AttrFcs[idx][5]

				if AttrFcs[idx][6]:
					mesh.faces[NewFcsIdx[idx]].materialIndex = AttrFcs[idx][6]


			mesh.update()
			Obj.makeDisplayList()

			print "BandGlue ...OK !"

	return 0

###################

###############
def ph_Timer():
###############
	global warn

	Draw()

	TheTime = 0.0

	while (TheTime < DelayWanted):
		end = time.clock()
		TheTime = end - start

	warn = False
	Draw()

	return 0


###########
def draw():
###########
	global WindowSize

	color1 = (0.99, 0.99, 0.65)
	color2 = (0.567, 0.476, 0.365)
	color2Clear = (0.567, 0.476, 0.365, 1)
	color3 = (1.0, 1.0, 0.79)
	color4 = (0.265, 0.41, 0.4)
	color4Clear = (0.265, 0.41, 0.4, 1)
	color5 = (0.4, 0.472, 0.472)
	color6 = (0.5, 0.25, 0.25)
	color7 = (0.265, 0.4, 0.3)
	colorblack = (0.0, 0.0, 0.0)
	colorwhite = (1.0, 1.0, 1.0)
	colorred = (1.0, 0.0, 0.0)
	colorcyan = (0.0, 1.0, 1.0)
	colorgrey1 = (0.8, 0.8, 0.8)

	WindowSize = Buffer(GL_FLOAT, 4)
	glGetFloatv(GL_SCISSOR_BOX, WindowSize)
	WindowSize = WindowSize.list
#	print WindowSize

	apply(glClearColor, color4Clear)
	glClear(GL_COLOR_BUFFER_BIT)

########

	Degrade(GL_By(- 1), BHeight - 2, (0.8, 0.8, 0.6), 20,GL_Bx(1),GL_Bx(BWidth), -0.4)

	if warn:
		apply(glColor3f, colorblack)
		glRecti(GL_Bx(1), GL_By(- 1), GL_Bx(BWidth), GL_By(BHeight - 3))
		apply(glColor3f, colorgrey1)
		glRecti(GL_Bx(), GL_By(1), GL_Bx(BWidth - 1), GL_By(BHeight - 2))
		apply(glColor3f, colorred)
		glRecti(GL_Bx(1), GL_By(1), GL_Bx(BWidth - 1), GL_By(BHeight - 3))


	Degrade(GL_By(BHeight -39), 35, (0.85, 0.7, 0.5), 35,GL_Bx(1),GL_Bx(BWidth - 1), 0.00)
	Degrade(GL_By(BHeight -39), 35, (0.85, 0.7, 0.5), 60,GL_Bx(1),GL_Bx(100), 0.44,0,1)
	Relief_Text(GL_Bx(10), GL_By(BHeight - 28),"ph_BandGlue.py", (0.3,0.3,0.3), (0.962, 0.792, 0.566))


	GL_DotLine(GL_Bx(10), GL_By(10), 80)
	GL_DotLine(GL_Bx(BWidth - 10), GL_By(10), 80)


	Ligne3D(GL_By(10),50,(0.8, 0.8, 0.6),-30,GL_Bx(20))
	Ligne3D(GL_By(BHeight - 50),35,(0.8, 0.8, 0.6),-30,GL_Bx(BWidth - 55))

	GL_B_4dir(BWidth - 94,BHeight - 30)

#########

	Button("Exit", EVENT_EXIT, GL_Bx(BWidth - 60), GL_By(BHeight - 30), 50, 20, "Quit")

	Button("BANDGLUE", EVENT_ACTION, GL_Bx(65), GL_By(50), 100, 30, "BandGlue selected faces.")

	Button("Undo last", EVENT_UNDO, GL_Bx(82), GL_By(15), 65, 22, "Undo last action.")

####################
def event(evt, val):
####################
	if ( evt == ESCKEY and not val):
		Exit()

	if (evt== QKEY and not val):
		Exit()



################
def bevent(evt):
################

	if (evt == EVENT_EXIT):
		Exit()


####### Position fenetre #######
	if (evt == EVENT_WPOS_U):
		if W_up.val:
			W_down.val = 0
		else:
			W_down.val = 1
		Draw()

	if (evt == EVENT_WPOS_D):
		if W_down.val:
			W_up.val = 0
		else:
			W_up.val = 1
		Draw()

	if (evt == EVENT_WPOS_L):
		if W_left.val:
			W_right.val = 0
		else:
			W_right.val = 1
		Draw()

	if (evt == EVENT_WPOS_R):
		if W_right.val:
			W_left.val = 0
		else:
			W_left.val = 1
		Draw()

################################


	if (evt == EVENT_ACTION):
		main()

	if (evt == EVENT_UNDO) and lastUndo:
		Undo()

	if warn:
		ph_Timer()

	Draw()


######################
Register(draw, event, bevent)
######################